!> \file dragging_calc_beta_mod.f90
!! Module qui calcule le beta a partir de vitesses de bilan
!<

!> \namespace  dragging_calc_beta
!! Calcule le beta a partir de vitesses de bilan
!! @note Il faut partir d'un cptr.
!! \author Cat
!! \date august 2010
!! @note Used module
!! @note   - use module3D_phy
!<

module dragging_calc_beta

  ! Calcule le beta a partir de vitesses de bilan
  ! il faut partir d'un cptr (pour avoir un bon champ de temperature)



  use module3d_phy
  use interface_input

  implicit none

  logical, dimension(nx,ny) :: gz_centre     !< stream on major node
  real, dimension(nx,ny) :: Vcol_x           !< vertically averaged velocity along x (balance)
  real, dimension(nx,ny) :: Vcol_y           !< vertically averaged velocity along y (balance)
  
  real, dimension(nx,ny) :: Vsl_x            !<sliding velocity x direction (balance)
  real, dimension(nx,ny) :: Vsl_y            !<sliding velocity y direction (balance)

  real, dimension(nx,ny) :: Vcol2            !<  square of vertically averaged velocity norme  on major nodes (balance)
!  real, dimension(nx,ny) :: beta_centre     ! beta on major node (average) declare dans 3D
  real :: beta_limgz                         !< when beta gt beta_limgz -> not gzmx
  real :: beta_bord                          !< for the sides of ice streams
  real :: ubil_limgz                         !< when ubil < ubil_limgz  -> not gzmx
  real :: coefbeta                           !< coefficient to ajust beta
  real :: ecart_quad                         !< somme of quadratic difference 
  real :: umag2                              !< square of veloc. at major node
  integer :: stagger= 0                      !< iteration based on centered (stagger = 0) 
                                             !< stagg nodes (stagger = 1)
                                             !< or redistributed (stagger = 2)
  logical :: corr_def = .true.               !< for deformation correction


contains
!-----------------------------------------------------------------------------------
  subroutine init_dragging      ! Cette routine fait l'initialisation du dragging.

    implicit none

    if (itracebug.eq.1)  call tracebug(' Calc_beta       subroutine init_dragging')
    mstream_mx(:,:)=1            ! autorise streams partout 
    mstream_my(:,:)=1

    beta_limgz=5.e5              ! au dela duquel on considere pas de glissement
    beta_bord= 1000.             ! pour regler des problemes a la cote         
    ubil_limgz=.1 

    ! coefficient permettant de modifier le basal drag.
    drag_mx(:,:)=1.
    drag_my(:,:)=1.

    call lect_vitbil

    iter_beta = 1
    iter_loop : if (iter_beta.eq.1) then  ! pour calculer initial guess de beta a partir de vitesses

       betamx(:,:)=0
       betamy(:,:)=0
       beta_centre(:,:)=0.
       coefbeta=5.
       betamax = beta_limgz

       call gzm_betacalc                  ! determine flgzmx, ...

    else if (iter_beta.eq.0) then         ! pour calculer les vitesses dynamiques deduites de iter_beta=1
       call gzm_betacalc                  ! determine flgzmx, ...

       betamx(:,:) = max(0., betamx(:,:)) ! enleve les beta negatifs
       betamy(:,:) = max(0., betamy(:,:))

    else if (iter_beta.eq.2) then     ! pour partir d'un beta uniforme,  (ne marche pas)

       betamx(:,:)=0                  ! pour les tests gzm_betaclac 
       betamy(:,:)=0

       call gzm_betacalc              ! determine flgzmx, ...

       betamx(:,:)=1.e5
       betamy(:,:)=1.e5
       beta_centre(:,:)=0.
       coefbeta=10.

    else if (iter_beta.eq.3) then      ! iteration Arthern en partant d'un champ lu

       call lect_beta_stag
       call average_beta_centr
       call distribute_stag_beta       !  averaging with minimum value
       call gzm_betacalc               ! determine flgzmx, ...

       coefbeta=1.e-3

    end if iter_loop

    return

  end subroutine init_dragging


  !-------------------------------------------------------------------------
  subroutine dragging   ! defini la localisation des streams et le frottement basal

    implicit none


    if (itracebug.eq.1)  call tracebug(' Subroutine dragging calc beta')
    if (itracebug.eq.1) write(num_tracebug,*)' Dragging_calc_beta: iter=',iter_beta, &
                        '   corr_def=',corr_def
    call gzm_betacalc   


    if (iter_beta.eq.1) then 
       betamx(:,:)=0.
       betamy(:,:)=0.

    else if ((iter_beta.eq.0).and.(corr_def)) then
       call correct_from_def
       call gzm_betacalc   

    else if (iter_beta.gt.1) then  ! boucle iterative
 
       call Arthern_like_iter     ! update betamx and betamy
       iter_beta=iter_beta+1
    end if

   

    if (itracebug.eq.1)  call tracebug(' Dragging sortie')

    return
  end subroutine dragging

  !________________________________________________________________________________

  subroutine lect_vitbil


    character(len=100) :: balance_Ux_file        !  balance velocity file Ux
    character(len=100) :: balance_Uy_file        !  balance velocity file Uy

    namelist/vitbil_upwind/balance_Ux_file, balance_Uy_file 

    if (itracebug.eq.1)  call tracebug(' Subroutine lect_vitbil de calc_beta') 

    ! lecture des parametres du run                    
    !--------------------------------------------------------------------

    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
428 format(A)
    read(num_param,vitbil_upwind)

    write(num_rep_42,428) '!___________________________________________________________' 
    write(num_rep_42,428) '!read balance velocities on staggered grid'
    write(num_rep_42,vitbil_upwind)                    
    write(num_rep_42,428) '! balance_Ux_file : nom du fichier qui contient les vit. bilan Ux'
    write(num_rep_42,428) '! balance_Uy_file : nom du fichier qui contient les vit. bilan Uy'
    write(num_rep_42,*)

    ! read balance velocities on staggered nodes
    
    balance_Ux_file = trim(dirnameinp)//trim(balance_Ux_file)
    balance_Uy_file = trim(dirnameinp)//trim(balance_Uy_file)

    call lect_input(1,'Vcol_x',1,Vcol_x,balance_Ux_file,"")
    call lect_input(1,'Vcol_y',1,Vcol_y,balance_Uy_file,"")

    ! limit the maximum value

    Vcol_x(:,:)=max(-3900.,Vcol_x(:,:))
    Vcol_x(:,:)=min( 3900.,Vcol_x(:,:))
    Vcol_y(:,:)=max(-3900.,Vcol_y(:,:))
    Vcol_y(:,:)=min( 3900.,Vcol_y(:,:))


    do j=2,ny-1
       do i=2,nx-1
          umag2=((Vcol_x(i,j)+Vcol_x(i+1,j))*(Vcol_x(i,j)+Vcol_x(i+1,j))) &  ! ux2
               +((Vcol_y(i,j)+Vcol_y(i,j+1))*(Vcol_y(i,j)+Vcol_y(i,j+1)))    ! uy2
          Vcol2(i,j) =umag2*0.25
       end do
    end do


    debug_3D(:,:,59)=Vcol_x(:,:)
    debug_3D(:,:,60)=Vcol_y(:,:)
    debug_3D(:,:,58)=Vcol2(:,:)**0.5

  end subroutine lect_vitbil
!__________________________________________________________________________________________
  subroutine lect_beta_stag


    character(len=100) :: betamx_file        ! betamx file 
    character(len=100) :: betamy_file        ! betamy file
    
    namelist/beta_stag/betamx_file,betamy_file,beta_limgz
    
    if (itracebug.eq.1)  call tracebug(' Subroutine lect__beta_stag de calc_beta') 
    
    ! lecture des parametres du run                    
    !--------------------------------------------------------------------

    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
428 format(A)
    read(num_param,beta_stag)

    write(num_rep_42,428) '!___________________________________________________________' 
    write(num_rep_42,428) '!read beta on staggered grid'
    write(num_rep_42,beta_stag)                    
    write(num_rep_42,428) '! betamx_file : nom du fichier qui contient les betamx'
    write(num_rep_42,428) '! betamy_file : nom du fichier qui contient les betamy'
    write(num_rep_42,428) '! above beta_limgz, gzmx is false'
    write(num_rep_42,*)


    call lect_input(1,'betamx',1,betamx,betamx_file,"")
    call lect_input(1,'betamy',1,betamy,betamy_file,"")

! average on major nodes

    beta_centre(:,:)=0.
    do j=2,ny-1
       do i=2,nx-1
          beta_centre(i,j)= ((betamx(i,j)+betamx(i+1,j)) &
               + (betamy(i,j)+betamy(i,j+1)))*0.25
       end do
    end do

  end subroutine lect_beta_stag
!____________________________________________________________________________________
subroutine average_beta_centr          ! average betamx and betamy on major nodes

do j=2,ny-1
   do i=2,nx-1
      beta_centre(i,j) = (betamx(i,j) + betamx(i+1,j)) + (betamy(i,j) + betamy(i,j+1))
      beta_centre(i,j) =  beta_centre(i,j) * 0.25
   end do
end do
end subroutine average_beta_centr

!-----------------------------------------------------------------------
subroutine distribute_stag_beta        ! redistribute on staggered grid


if (stagger.eq.0) then       !  MIN(betamx,average beta_centr)
   do j=2,ny
      do i=2,nx
         betamx(i,j)= min(betamx(i,j),(beta_centre(i-1,j)+beta_centre(i,j))*0.5)
         betamy(i,j)= min(betamx(i,j),(beta_centre(i,j-1)+beta_centre(i,j))*0.5)
      end do
   end do
else if (stagger.eq.1) then       ! usual average
   do j=2,ny
      do i=2,nx
         betamx(i,j)=(beta_centre(i-1,j)+beta_centre(i,j))*0.5
         betamy(i,j)=(beta_centre(i,j-1)+beta_centre(i,j))*0.5
      end do
   end do
   
else if (stagger.eq.2) then  ! MIN(betamx,beta_bord) only on ice streams V> 50 m/year
   
   do j=2,ny
      do i=2,nx
         if (abs(Vcol_x(i,j)).gt.50.) then     ! fleuve de glace
            betamx(i,j+1)=min(betamx(i,j+1),beta_bord)
            betamx(i,j-1)=min(betamx(i,j-1),beta_bord)
            betamx(i-1,j)=min(betamx(i-1,j),beta_bord)
            betamx(i+1,j)=min(betamx(i+1,j),beta_bord)
         end if
         if (abs(Vcol_y(i,j)).gt.50.) then     ! fleuve de glace
            betamy(i+1,j)=min(betamy(i+1,j),beta_bord)
            betamy(i-1,j)=min(betamy(i-1,j),beta_bord)
            betamy(i,j+1)=min(betamy(i,j+1),beta_bord)
            betamy(i,j-1)=min(betamy(i,j-1),beta_bord)
         end if
      end do
   end do
end if 

end subroutine distribute_stag_beta

!----------------------------------------------------------------------------------
subroutine correct_from_def             

! update la cible vitesse pour enlever la deformation 

if (itracebug.eq.1)  call tracebug(' Subroutine correct_from_def')

!Vsl_x(:,:) = Vcol_x(:,:) - uxdef(:,:)
!Vsl_y(:,:) = Vcol_y(:,:) - uydef(:,:)

do j = 2, ny
   do i = 2, nx
      if (Vcol_x(i,j).gt.0.) then
         Vsl_x(i,j) = max(0.,Vcol_x(i,j) - uxdef(i,j))   ! si la vitesse de deformation est > vitesse de bilan : pas de glissement

      else if (Vcol_x(i,j).lt.0.) then
         Vsl_x(i,j) = min(0.,Vcol_x(i,j) - uxdef(i,j)) 
      else
         Vsl_x(i,j) = 0.
      endif

      if (Vcol_y(i,j).gt.0.) then
         Vsl_y(i,j) = max(0.,Vcol_y(i,j) - uydef(i,j))   ! si la vitesse de deformation est > vitesse de bilan : pas de glissement

      else if (Vcol_y(i,j).lt.0.) then
         Vsl_y(i,j) = min(0.,Vcol_y(i,j) - uydef(i,j)) 
      else
         Vsl_y(i,j) = 0.
      endif

   end do
end do

do j = 2, ny-1
   do i = 2, nx-1
      debug_3D(:,:,68) = (((Vsl_x(i,j)+Vsl_x(i+1,j))**2 + &
                           (Vsl_y(i,j)+Vsl_y(i,j+1))**2)*0.25)**0.5 
      debug_3D(:,:,69) =  Vsl_x(i,j)
      debug_3D(:,:,70) =  Vsl_y(i,j)
   end do
end do

end subroutine correct_from_def

subroutine gzm_betacalc

    ! calcul de gzmx qui ne sera plus modifie ensuite
if (itracebug.eq.1)  call tracebug(' Subroutine  gzm_betacalc')

    gzmx(:,:)=.true.
    gzmy(:,:)=.true.
    flgzmx(:,:)=.false.
    flgzmy(:,:)=.false.


    ! points flottants : flgzmx mais pas gzmx
    do j=2,ny
       do i=2,nx
          if (flot(i,j).and.(flot(i-1,j))) then
             flgzmx(i,j)=.true.
             gzmx(i,j)=.false.
             betamx(i,j)=0.
             flotmx(i,j) = .true.
          end if
          if (flot(i,j).and.(flot(i,j-1))) then
             flgzmy(i,j)=.true.
             gzmy(i,j)=.false.
             betamy(i,j)=0.
             flotmy(i,j) = .true.
          end if
       end do
    end do

where (flot(:,:))
   beta_centre(:,:)= 0.
end where


    !--------- autres criteres

    ! points poses

    ! limite le calcul elliptique aux points a faible beta
!    gzmx(:,:)=gzmx(:,:).and.(betamx(:,:).lt.beta_limgz)   !  Pas de calcul pour les points
!    gzmy(:,:)=gzmy(:,:).and.(betamy(:,:).lt.beta_limgz)   !  au fort beta

    ! pour tre sr d'avoir quelques points "Dirichlet"
    gzmx(:,:)=gzmx(:,:).and.(abs(Vcol_x(:,:)).gt.ubil_limgz) ! Pas de calcul pour les pts lents
    gzmy(:,:)=gzmy(:,:).and.(abs(Vcol_y(:,:)).gt.ubil_limgz) ! Pas de calcul pour les pts lents


    flgzmx(:,:)=flgzmx(:,:).or.gzmx(:,:)
    flgzmy(:,:)=flgzmy(:,:).or.gzmy(:,:)

    gz_centre(:,:)=.false.
    do j=2,ny-1
       do i=2,nx-1
          gz_centre(i,j)=gzmx(i,j).or.gzmx(i+1,j).or.gzmy(i,j).or.gzmy(i,j+1)
       end do
    end do

    where (flgzmy(:,:))
       debug_3D(:,:,84)=1
    elsewhere
       debug_3D(:,:,84)=0
    endwhere

  end subroutine gzm_betacalc
!____________________________________________


 subroutine  Arthern_like_iter   ! ATTENTION CE BLOC A ETE DEPLACE ET PAS REVALIDE

   ! iteration Arthern like sur beta
   !________________________________________
   ! On part d un beta issus de calc beta et eventuellement lisse par moyenne mobile
   !
   ! Vcol_x tient le role de solution Dirichlet et uxbar de solution Neumann


   ecart_quad=0.
   if (iter_beta.ge.4) then
      
      stag: if (stagger.eq.1) then
         do j=2,ny
            do i=2,nx
               if (gzmx(i,j)) then
                  betamx(i,j)=betamx(i,j)-coefbeta*(Vcol_x(i,j)*Vcol_x(i,j)-uxbar(i,j)*uxbar(i,j))
                  betamx(i,j)=max(betamx(i,j),0.)
                  ecart_quad= ecart_quad  & 
                       + abs(Vcol_x(i,j)*Vcol_x(i,j)-uxbar(i,j)*uxbar(i,j))
               end if
               if (gzmy(i,j)) then
                  betamy(i,j)=betamy(i,j)-coefbeta*(Vcol_y(i,j)*Vcol_y(i,j)-uybar(i,j)*uybar(i,j))
                  betamy(i,j)=max(betamy(i,j),0.)
                  ecart_quad= ecart_quad  & 
                       + abs(Vcol_y(i,j)*Vcol_y(i,j)-uybar(i,j)*uybar(i,j))
               end if
            end do
         end do
      else if (stagger.eq.0) then          ! le chemin se fait sur les noeuds centres
         do j=2,ny-1
            do i=2,nx-1
               if (gz_centre(i,j)) then
                  umag2=((uxbar(i,j)+uxbar(i+1,j))*(uxbar(i,j)+uxbar(i+1,j))) &  ! ux2
                       +((uybar(i,j)+uybar(i,j+1))*(uybar(i,j)+uybar(i,j+1)))    ! uy2
                  umag2=umag2*0.25
                  
                  beta_centre(i,j)=beta_centre(i,j)-coefbeta*(Vcol2(i,j)-umag2) 
                  beta_centre(i,j)=max(beta_centre(i,j),0.)            
                  ecart_quad= ecart_quad  & 
                       + abs(Vcol2(i,j)-umag2) 
               end if
            end do
         end do
         ! redistribute on staggered grid
         do j=2,ny
            do i=2,nx
               betamx(i,j)= (beta_centre(i-1,j)+beta_centre(i,j))*0.5
               betamy(i,j)= (beta_centre(i,j-1)+beta_centre(i,j))*0.5
            end do
         end do

      else if (stagger.eq.2) then          ! le chemin se fait sur les noeuds centres
         beta_centre(:,:)=0.
         write(6,*) coefbeta
         do j=2,ny-1
            do i=2,nx-1
               if (gz_centre(i,j)) then
                  umag2=((uxbar(i,j)+uxbar(i+1,j))*(uxbar(i,j)+uxbar(i+1,j))) &  ! ux2
                       +((uybar(i,j)+uybar(i,j+1))*(uybar(i,j)+uybar(i,j+1)))    ! uy2
                  umag2=umag2*0.25
                  
                  beta_centre(i,j)=-coefbeta*(Vcol2(i,j)-umag2) 
                  beta_centre(i,j)=min(beta_centre(i,j),100.)
                  beta_centre(i,j)=max(beta_centre(i,j),-100.)
                  ecart_quad= ecart_quad  & 
                       + abs(Vcol2(i,j)-umag2) 
               end if
            end do
         end do
         debug_3D(:,:,86)=beta_centre(:,:)

         ! redistribute on staggered grid
         do j=2,ny
            do i=2,nx
               betamx(i,j)= betamx(i,j)+((beta_centre(i-1,j)+beta_centre(i,j)) &
                    + (beta_centre(i-1,j-1)+beta_centre(i,j-1)) &
                    + (beta_centre(i-1,j+1)+beta_centre(i,j+1)))/6. 
               
               betamx(i,j)=max(betamx(i,j),0.)
               betamx(i,j)=min(betamx(i,j),beta_limgz)
               
               betamy(i,j)= betamy(i,j)+((beta_centre(i,j-1)+beta_centre(i,j)) &
                    + (beta_centre(i-1,j-1)+beta_centre(i-1,j)) &
                    + (beta_centre(i+1,j-1)+beta_centre(i+1,j)))/6. 
               
               betamy(i,j)=max(betamy(i,j),0.)
               betamy(i,j)=min(betamy(i,j),beta_limgz)
            end do
         end do
         
      else if (stagger.eq.-1) then  ! test dramatique !
         write(6,*) 'stagger', stagger, iter_beta
         do j=2,ny
            do i=2,nx
               if (abs(uxbar(i,j))-abs(Vcol_x(i,j)).lt.-50.) then
                  !             write(6,*) i,j, betamx(i,j),betamx(i,j+1), betamx(i,j-1),uxbar(i,j),Vcol_x(i,j)
                  betamx(i,j)=min(betamx(i,j),100.)
                  !             write(6,*) i,j, betamx(i,j),betamx(i,j+1), betamx(i,j-1)
               endif
               
               if (abs(uybar(i,j))-abs(Vcol_y(i,j)).lt.-50.) then
                  betamy(i,j)=min(betamy(i,j),100.)
               endif
            end do
         end do
         do j=2,ny
            do i=2,nx
               if (betamx(i,j).eq.-1) then
                  
                  betamx(i,j)=100.
                  betamx(i,j+1)=200
                  betamx(i,j-1)=200
               end if
               if (betamy(i,j).eq.-1) then
                  betamy(i,j)=100
                  betamy(i+1,j)=200
                  betamy(i-1,j)=200
               end if
            end do
         end do
      endif stag
   else
      do j=2,ny
         do i=2,nx
            if (gzmx(i,j)) then
               ecart_quad= ecart_quad  & 
                    + abs(Vcol_x(i,j)*Vcol_x(i,j)-uxbar(i,j)*uxbar(i,j))
            end if
            if (gzmy(i,j)) then
               ecart_quad= ecart_quad  & 
                    + abs(Vcol_y(i,j)*Vcol_y(i,j)-uybar(i,j)*uybar(i,j))
            end if
         end do
      end do
   end if


   write(6,*) 'ecart_quad',ecart_quad
   !-------------------------- fin iteration Arthern like--------------------------
 end subroutine Arthern_like_iter
 

end module dragging_calc_beta

